from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QApplication, QShortcut
from PyQt5.QtGui import QKeySequence


def setCloseOnEscKey(window):
    window.closeOnEscShortcut = QShortcut(QKeySequence(Qt.Key_Escape), window)
    window.closeOnEscShortcut.activated.connect(lambda: QApplication.quit())


def setOpenOnOKey(window, openLambda):
    window.openOnOShortcut = QShortcut(QKeySequence(Qt.Key_O), window)
    window.openOnOShortcut.activated.connect(openLambda)



from PyQt5.QtCore import Qt, pyqtSlot, QSize
from PyQt5.QtWidgets import QMainWindow, QSizePolicy, QWidget, QFrame, QLabel, QGridLayout
from PyQt5.QtGui import QPixmap

from ezdmb.Utility.ShortcutUtility import setCloseOnEscKey, setOpenOnOKey
from ezdmb.View import MenuContentViewUtility
from ezdmb.Controller.Configuration import Configuration


class FullScreenWindow(QMainWindow):
    def __init__(self, config: Configuration, reopenPreviewWindow):
        super(self.__class__, self).__init__()
        self._config = config

        self.setObjectName("FullScreenWindow")
        self.setWindowModality(Qt.WindowModal)
        self.setEnabled(True)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        self.setSizePolicy(sizePolicy)
        self.setMinimumSize(QSize(200, 200))
        self.setAutoFillBackground(True)
        self.centralwidget = QWidget(self)
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setAutoFillBackground(True)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QGridLayout(self.centralwidget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.gridLayout_2 = QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.contentLbl = QLabel(self.centralwidget)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        self.contentLbl.setSizePolicy(sizePolicy)
        self.contentLbl.setFrameShape(QFrame.NoFrame)
        self.contentLbl.setLineWidth(0)
        self.contentLbl.setTextFormat(Qt.RichText)
        self.contentLbl.setScaledContents(True)
        self.contentLbl.setObjectName("contentLbl")
        self.gridLayout_2.addWidget(self.contentLbl, 0, 0, 1, 1)
        self.gridLayout.addLayout(self.gridLayout_2, 0, 0, 1, 1)
        self.setCentralWidget(self.centralwidget)
        self.setWindowTitle("Menu")

        self.contentViewUtil = MenuContentViewUtility.MenuContentViewUtility(
            self._config,
            self.contentLbl,
            "FullScreenWindow",
            self.onRefresh,
        )
        setCloseOnEscKey(self)
        setOpenOnOKey(self, reopenPreviewWindow)

    @pyqtSlot(QPixmap)
    def onRefresh(self, value):
        self.contentLbl.setPixmap(value)



import os
import errno
import json
from PyQt5.QtCore import pyqtSignal, QObject


# Encapsulates configuration file serialization and deserialization
class Configuration(QObject):
    # signals
    configUpdated = pyqtSignal(dict)

    # properties

    def set_data(self, value):
        self._data = value

    def get_data(self):
        return self._data

    Data = property(get_data, set_data)

    def set_use_images(self, value):
        self._use_images = value

    def get_use_images(self):
        return self._use_images

    UseImages = property(get_use_images, set_use_images)

    def set_use_html(self, value):
        self._use_html = value

    def get_use_html(self):
        return self._use_html

    UseHTML = property(get_use_html, set_use_html)

    def set_use_imported(self, value):
        self._use_imported = value

    def get_use_imported(self):
        return self._use_imported
    UseImported = property(get_use_imported, set_use_imported)

    def set_rotate_content(self, value):
        self._rotate_content = value

    def get_rotate_content(self):
        return self._rotate_content

    RotateContent = property(get_rotate_content, set_rotate_content)

    def set_rotate_content_time(self, value):
        self._rotate_content_time = value

    def get_rotate_content_time(self):
        return self._rotate_content_time

    RotateContentTime = property(get_rotate_content_time, set_rotate_content_time)

    def set_content_array(self, value):
        self._content_array = value

    def get_content_array(self):
        return self._content_array

    ContentArray = property(get_content_array, set_content_array)

    # Functions

    # Initializes the object when Configuration is first instanced
    def __init__(self):
        super(self.__class__, self).__init__()
        flags = os.O_CREAT | os.O_RDWR
        self._data = dict()
        self._configPath = "dmb_config.json"
        # Create config file if it does not already exist
        if not os.path.exists(self._configPath):
            try:
                os.open(self._configPath, flags)
            except OSError as e:
                if e.errno == errno.EEXIST:  # Failed as the file already exists.
                    pass
                else:  # Something unexpected went wrong so reraise the exception.
                    raise
            else:  # No exception, so the file must have been created successfully.
                # Settings defaults here, for first runs
                self.SaveConfig(True, "15", [])

        with open(self._configPath, "r+") as json_data_file:
            self._data = json.load(json_data_file)

        # Set variables for the app to use
        self.set_rotate_content(self._data["rotate_content"])
        self.set_rotate_content_time(int(self._data["rotate_content_time"]))
        self.set_content_array(self._data["imported_content"])

    # Saves our JSON config file
    def SaveConfig(
        self,
        RotateContent,
        RotateContentTime,
        ContentArray,
    ):
        self._data["rotate_content"] = RotateContent
        self._data["rotate_content_time"] = int(RotateContentTime)
        self._data["imported_content"] = ContentArray
        with open(self._configPath, "w+") as outfile:
            json.dump(self.get_data(), outfile)

        self.configUpdated.emit(self._data)



from PyQt5.QtWidgets import (
    QDialog,
    QDialogButtonBox,
    QLabel,
    QVBoxLayout,
)

from ezdmb import __version__


class AboutDialog(QDialog):
    def __init__(self):
        super().__init__()

        self.setWindowTitle("About ezdmb")

        message = QLabel("ezdmb v" + __version__ + "\nGithub: https://github.com/justinmichaelvieira/ezdmb\n")

        self.buttonBox = QDialogButtonBox(QDialogButtonBox.Close)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)

        layout = QVBoxLayout()
        layout.addWidget(message)
        layout.addWidget(self.buttonBox)
        self.setLayout(layout)



# main.py
# DMB startup script
# Justin Vieira [justin@rancorsoft.com] / Richard Haynes / Adam Brody
# Rancorsoft, LLC

import logging
import sys

from PyQt5 import QtCore, QtGui
from PyQt5.QtWidgets import QApplication

from ezdmb.Controller import Configuration
from ezdmb.Controller.LoggingUtility import setupLogging
from ezdmb.View import AboutDialog, FullScreenWindow, ConfigDialog, PreviewWindow

_styleSheet = """
* {
    border-color: #2b2b2b;
    font-size: 13px;
    alternate-background-color: #3c3f41;
}

QPushButton::pressed {
    background: #303F9F;
    color: #448AFF;
}

QToolButton {
    color: #FFFFFF;
}

QLineEdit[accessibleName="selectedFileEdit"] {
    color: #757575;
}

QScrollArea {
    background: #ffffff;
}

QToolButton {
    background: #303F9F;
    border: 0;
}

QFrame[accessibleName="navHeaderFrame"] {
    background: #303F9F;
}

QLabel[accessibleName="titleBar"] {
    background: #3F51B5;
    color: #FFFFFF;
}

QFrame {
    border: none;
}

QScrollArea {
    color: #bbbbbb;
    background-color: #3c3f41;
    border: none;
    border-top: 1px solid #2b2b2b;
    selection-background-color: #2f65ca;
    selection-color: #bbbbbb;
}
QDialog QScrollArea {
    border-top: none;
    border: none;
}

QPlainTextEdit {
    background-color: #2b2b2b;
    border: none;
    color: #bbbbbb;
    selection-background-color: #2f65ca;
}

QGraphicsView {
    background-color: #3c3f41;
    border-color: #2b2b2b;
    color: #bbbbbb;
}
"""
_logger = logging.getLogger()


# starting point of the app runtime
def main():
    app, fullScreenMenu, _advancedConfig, _mainwin = populateInstance()
    setupLogging()
    # store screen geometry
    screenWidth = fullScreenMenu.frameGeometry().width()
    screenHeight = fullScreenMenu.frameGeometry().height()
    # size and show menu
    fullScreenMenu.contentLbl.resize(screenWidth, screenHeight)
    # without this, the script exits immediately.
    _logger.info("DMB Application started.")
    sys.exit(app.exec_())


def populateInstance():
    app = QApplication(sys.argv)
    app.setOrganizationName("Rancorsoft")
    app.setOrganizationDomain("Rancorsoft.com")
    app.setApplicationName("Digital Menu Board")

    app.setStyleSheet(_styleSheet)

    _aboutWin = AboutDialog.AboutDialog()

    _config = Configuration.Configuration()
    _configWin = ConfigDialog.ConfigDialog(_config)

    def showConfig():
        _configWin.show()

    def showAboutWindow():
        _aboutWin.show()

    _previewWin = PreviewWindow.PreviewWindow(_config, showConfig, showAboutWindow)
    _previewWin.setWindowIcon(QtGui.QIcon(":/logo_256x256.jpg"))

    def openPreviewWindow():
        showAndBringToFront(_previewWin)

    _fullScreenWin = FullScreenWindow.FullScreenWindow(_config, openPreviewWindow)

    _fullScreenWin.setWindowFlags(QtCore.Qt.FramelessWindowHint)
    _fullScreenWin.showFullScreen()

    openPreviewWindow()
    _previewWin.raise_()
    _previewWin.activateWindow()
    return app, _fullScreenWin, _configWin, _previewWin


def showAndBringToFront(window):
    window.show()
    window.raise_()
    window.activateWindow()


if __name__ == "__main__":
    main()
